/* V1.00 2018-APR-20  GJO
 *  
 * QE Encoder Program to test Peter Daneggers QE Encoder Algorithm
 *  https://www.mikrocontroller.net/attachment/highlight/40597
 * 
 * HW: PRO-MINI at 16MHz and 5V
 *     KY-040 Mechanical QE-Encoder Board (30 steps)
 * 
*/

/*

      Testing Hardware Configuration:

                           Pro Mini Board/Nano
                            (328P/16MHZ/5V)

                          +-DTR TX RX Vc G G-+
                         1| TX           VIN |24    +12V INPUT
                          | RX           GND |      (AVRISP:GND   _ PIN 6)
                          | RST          RST |      (AVRISP:RESET _ PIN 5)
                          | GND          VCC |      (AVRISP:VCC   _ PIN 2)
                          | D2     (A4)  A3  |
                          | D3     (A5)  A2  |
                          | D4           A1  |
                          | D5           A0  |
  QUADR-PHASE A           | D6           D13 |o     (AVRISP:SCK  _ PIN 3)
  QUADR-PHASE B          o| D7           D12 |o     (AVRISP:MISO _ PIN 1)
                         o| D8           D11 |      (AVRISP:MOSI _ PIN 4)
                        12| D9           D10 |13
                          +--G-A4 A5-A6 A7 --+

  QE Encoder used for testing:
  http://henrysbench.capnfatz.com/henrys-bench/arduino-sensors-and-input/keyes-ky-040-arduino-rotary-encoder-user-manual/

  Notes:
  TIMER0 is claimed by the Arduino framework, hence TIMER2 is used for the demo.

*/

//******************************************************************************
//****************        Include header files      ****************************
//******************************************************************************

// none

//*******************************************************************************
//**************************** Prototypes ***************************************
//*******************************************************************************

// none required

//*******************************************************************************
//**************************** CFG defines **************************************
//*******************************************************************************

#define QA_PIN    6
#define QB_PIN    7
#define QE_POWER  8

#define BAUD_USART0 9600

//*******************************************************************************
//******************             Global Variables           *********************
//*******************************************************************************

int gl_enc_ctr=0;
int gl_QE_total=0;

//*******************************************************************************
//******************              Begin Program             *********************
//*******************************************************************************

void setup() {
 
    Serial.begin(BAUD_USART0);

    pinMode(QA_PIN, INPUT_PULLUP);  
    pinMode(QB_PIN, INPUT_PULLUP);  
    pinMode(QE_POWER, OUTPUT);    // Powers KY-040 QE Encoder board pullups
    digitalWrite(QE_POWER, HIGH); // Set Pin High for Encoder Pullups

    Serial.println("Peter Danegger Encoder Test");

    timer2_Start(); // Configure TIMER2 and interrupt
}


void loop() 
{
    gl_QE_total += encode_read();
    Serial.println(gl_QE_total);
    delay(1000);
}


//**********************************************************************************************
//******************************** Interrupt Functions *****************************************
//**********************************************************************************************

void timer2_Start(void)
{
    TCCR2A=0x00; // All compare OFF
    TCCR2B=0x04; // 16MHZ/128
    // Timer 0 Overflow rate = 1ms (256-125=131)
    TCNT2=131;
    // Timer(s)/Counter(s) Interrupt(s) initialization
    TIMSK2=0x01; // Enabled
}



/******************************************************************************
* Function:     ISR(TIMER2_OVF_vect)
* Description:  System Timebase and scheduler
* Parameters:   None
* Return Value: None
******************************************************************************/
ISR(TIMER2_OVF_vect)
{
    TCNT2 = 131;  // Reload for 1ms Tick
  
    /////////////////////////////////////////////////////////
    // Quadrature encoder handling
    // Based on Code example "ENCODE.C" by Peter Danegger
    // https://www.mikrocontroller.net/attachment/highlight/40597
    // See alse encode_read() below
    /////////////////////////////////////////////////////////
  
    static int last;
    int nw, diff;
  
    nw = 0;
  
    if ( PIND & ( 1 << QA_PIN ) ) 
        nw = 1;
    if ( PIND & ( 1 << QB_PIN ) ) 
        nw ^= 3;
  
    // convert gray to binary
    diff = last - nw;              // difference last - new
    if ( diff & 1 )                // bit 0 = value (1)
    {
        last = nw;                   // store new as next last
        gl_enc_ctr += (diff & 2) - 1;  // bit 1 = direction (+/-)
    }
    
}


//**********************************************************************************************
//********************************* Callable Functions *****************************************
//**********************************************************************************************


/******************************************************************************
* Function:     int encode_read(void)                                         
* Description:  totalizes quadrature encoder differences                      
* Parameters:   global                                                        
* Return Value: new encoder relative count                                    
******************************************************************************/
int encode_read(void)
{
    static int val=0;
    cli();
    val = gl_enc_ctr;
    gl_enc_ctr &= 1;
    val >>= 1;
    sei();
    return val;
}

